%This is a script to measure change in the #zero crossings at each iteration of the
%AM algorithm 
%       This file is part of the Matlab code provided for the following
%       reproducible paper:
%
%       Doru C. Balcan, Gowri Srinivasa, Matthew C. Fickus, Jelena Kovacevic,
%       "Guaranteeing Convergence of Iterative Skewed Voting Algorithms for Image Segmentation",
%       Applied and Computational Harmonic Analysis, vol. 33(2), Sep. 2012.
%                                                                           
%       This program is free software; you can redistribute it and/or modify it  
%       under the terms of the GNU General Public License as published by the     
%       Free Software Foundation; either version 2 of the License, or (at your    
%       option) any later version. This software is distributed in the hope that  
%       it will be useful, but without any warranty; without even the implied     
%       warranty of merchantability or fitness for a particular purpose.          
%       See the GNU General Public License for more details                       
%       (enclosed in the file COPYING).   
% 
%       GNU General Public License,
%       Copyright (C) 2008,
%       bimagicLab, Center for Bioimage Informatics (CBI)
%       Carnegie Mellon University,
%       Pittsburgh, PA 15213, USA.
%                                                                           
% COMMENTS
%
%       Authors: Gowri Srinivasa and Doru C. Balcan
%       Latest modifications: Dec. 22, 2011.

clear all;
close all;

Time=now;
ExportDirectory=datestr(Time,'yyyy_mm_dd_HH_MM_SS');
dos(sprintf('md %s',ExportDirectory));
dos(sprintf('copy AM_%s.m %s\\Code.m',datestr(Time,'yyyy_mm_dd'),ExportDirectory));

ImageData=double(imread('OriginalImage.tif','tif'));

ExportImage=ImageData;
ExportImage=ExportImage-min(min(ExportImage));
ExportImage=ExportImage/max(max(ExportImage));
ContrastExponent=2^(-1);
ExportImage=ExportImage.^ContrastExponent;
ExportImage=uint8(floor(2^8*ExportImage));

imwrite(ExportImage,sprintf('%s\\Original.png',ExportDirectory));

% Processing original image

CoarseImage=ImageData;
for k=1:2
    CoarseImage=0.25*(CoarseImage+circshift(CoarseImage,[1 0])+circshift(CoarseImage,[0 1])+circshift(CoarseImage,[1 1]));
    CoarseImage=CoarseImage(1:2:end,1:2:end);
end

[M,N]=size(CoarseImage);

[X,Y]=ndgrid([0:M/2-1 -M/2:-1],[0:N/2-1 -N/2:-1]); % Requires M,N even

VotingScale=2^4;
%VotingScale=2^2;
VotingFilter=exp(-(X.^2+Y.^2)/VotingScale.^2);
VotingFilter=VotingFilter/sum(sum(VotingFilter));
FFTOfVotingFilter=fft2(VotingFilter);

SkewScale=2^2;
SkewFilter=exp(-(X.^2+Y.^2)/SkewScale.^2);
SkewFilter=SkewFilter/sum(sum(SkewFilter));
FilteredCoarseImage=ifft2(fft2(CoarseImage).*fft2(SkewFilter));
VotingSkew=erf(FilteredCoarseImage-10);

ExportImage=VotingSkew;
ExportImage=ExportImage-min(min(ExportImage));
ExportImage=ExportImage/max(max(ExportImage));
ExportImage=uint8(floor(2^8*ExportImage));

imwrite(ExportImage,sprintf('%s\\VotingSkew.png',ExportDirectory));

NumberOfMasks=2^6;

% Mask Initialization

MaskIndices=ceil(NumberOfMasks*rand(M,N));
InitialRandCells = MaskIndices;
%load InitialRandCells_Run5.mat;
%MaskIndices=InitialRandCells_Run5;

Masks=zeros(M,N,NumberOfMasks);
MaskNeighbors=zeros(M,N);
CheckNeighbors=zeros(M,N);

for MaskIndex=1:NumberOfMasks;
    Masks(:,:,MaskIndex)=(MaskIndices==MaskIndex);
end

MinGrayLevel=floor(2^(-2)*2^8);
MaxGrayLevel=floor((1-2^(-2))*2^8);
GraySlope=floor((MaxGrayLevel-MinGrayLevel)/(NumberOfMasks-1));

ExportImage=uint8(double(MaskIndices>1).*(GraySlope*(MaskIndices-1)+MinGrayLevel));
imwrite(ExportImage,sprintf('%s\\A_%03d.png',ExportDirectory,0));

% Mask Evolution
FilteredOnes = ifft2(fft2(double(ones(M,N))).*FFTOfVotingFilter);
NumberOfMaskChanges=1;
WhileIndex=0;
Counter=1; 
while NumberOfMaskChanges~=0;
      WhileIndex=WhileIndex+1;
      Counter=Counter+1; 
      MaskNeighbors=zeros(M,N);
    for MaskIndex=1:NumberOfMasks
        CheckNeighbors = 1-Masks(:,:,MaskIndex);
                Masks(:,:,MaskIndex)=ifft2(fft2(double(Masks(:,:,MaskIndex))).*FFTOfVotingFilter);        
                MaskNeighbors = MaskNeighbors+(FilteredOnes-Masks(:,:,MaskIndex)).*(1-CheckNeighbors);
    end 
Masks(:,:,1)=Masks(:,:,1)-0.9*VotingSkew;   
[LargestVote,NewMaskIndices]=max(Masks,[],3); % LargestVote unused, introduced to placate Matlab
NumberOfMaskChanges=sum(sum(abs(MaskIndices-NewMaskIndices)));
tot_neighbors(WhileIndex)=sum(sum(MaskNeighbors));

    MaskIndices=NewMaskIndices;
    uniqueMasks=unique(MaskIndices(:));
    NumberOfMasks = length(uniqueMasks);
    Masks=zeros(M,N,NumberOfMasks);
    for MaskIndex=1:NumberOfMasks;
        Masks(:,:,MaskIndex)=(MaskIndices==uniqueMasks(MaskIndex));
    end

    ExportImage=uint8(double(MaskIndices>1).*(GraySlope*(MaskIndices-1)+MinGrayLevel));
    if mod(Counter,10)==0
        imwrite(ExportImage,sprintf('%s\\A_%03d.png',ExportDirectory,WhileIndex));
    end

    disp(sprintf('%d',NumberOfMaskChanges))
end

NeighbIndex=WhileIndex-1;
tot_neighbors = tot_neighbors(1:NeighbIndex);
figure; plot(1:length(tot_neighbors),tot_neighbors); axis tight;
title('Zero-crossings')
magic_number=log(0.0001+log(log(log(tot_neighbors)))-min(log(log(log(tot_neighbors)))));
figure; plot(1:length(tot_neighbors),magic_number); axis tight; title('Zero-crossings')

beep;
